﻿using System;
using System.Collections.Generic;
using Contrib.SecuredSocketsLayer.Models;
using Orchard;
using Orchard.Commands;
using Orchard.ContentManagement;

namespace Contrib.SecuredSocketsLayer.Commands {
    public class SecuredSocketsLayersCommand : DefaultOrchardCommandHandler {
        private readonly IOrchardServices _services;

        public SecuredSocketsLayersCommand(IOrchardServices services) {
            _services = services;
        }

        [OrchardSwitch]
        public bool AuthenticationEnabled { get; set; }
        [OrchardSwitch]
        public bool AdminEnabled { get; set; }
        [OrchardSwitch]
        public bool CustomEnabled { get; set; }
        [OrchardSwitch]
        public string Urls { get; set; }
        [OrchardSwitch]
        public string SecureHostName { get; set; }
        [OrchardSwitch]
        public string InsecureHostName { get; set; }

        [CommandName("site setting set ssl")]
        [CommandHelp("site setting set ssl /AuthenticationEnabled:true /AdminEnabled:true /CustomEnabled:true /Urls:<value> /SecureHostName:domain.com /InsecureHostName:secure.domain.com\r\n" +
            "\tSet the 'SSL' site settings. Urls example: /Urls:\"'mysite.com/a','mysite.com/b'\"")]
        [OrchardSwitches("AuthenticationEnabled,AdminEnabled,CustomEnabled,Urls,SecureHostName,InsecureHostName")]
        public void SetSSLInfo() {
            var settings = _services.WorkContext.CurrentSite.As<SslSettingsPart>();
            if (settings == null) {
                return;
            }

            if (!string.IsNullOrWhiteSpace(Urls)) {
                var comma = false;
                var urlList = new List<string>();
                try {
                    Urls = Urls.Trim();
                    while (Urls.Length != 0) {
                        var first = Urls[0];
                        if (first == ',' && comma) {
                            Urls = Urls.Substring(1);
                            comma = false;
                        }
                        else if (first == '\'' && !comma) {
                            int end = Urls.IndexOf('\'', 1);
                            if (end == -1) {
                                throw new ArgumentException("Invalid Urls");
                            }
                            urlList.Add(Urls.Substring(1, end - 1));
                            Urls = Urls.Substring(end + 1);
                            comma = true;
                        }
                        else {
                            throw new ArgumentException("Invalid Urls");
                        }
                    }
                    if (!comma)
                        throw new ArgumentException("Invalid Urls");
                    Urls = string.Join("\r\n", urlList);
                }
                catch(ArgumentException) {
                    Context.Output.WriteLine(T("'Urls' site setting invalid"));
                    return;
                }
            }

            if (string.IsNullOrWhiteSpace(Urls)) {
                Urls = null;
            }

            settings.AuthenticationEnabled = AuthenticationEnabled;
            settings.AdminEnabled = AdminEnabled;
            settings.CustomEnabled = CustomEnabled;
            settings.Urls = Urls;
            settings.SecureHostName = SecureHostName;
            settings.InsecureHostName = InsecureHostName;

            Context.Output.WriteLine(T("'AuthenticationEnabled' site setting set to '{0}'", AuthenticationEnabled));
            Context.Output.WriteLine(T("'AdminEnabled' site setting set to '{0}'", AdminEnabled));
            Context.Output.WriteLine(T("'CustomEnabled' site setting set to '{0}'", CustomEnabled));
            Context.Output.WriteLine(T("'Urls' site setting set to '{0}'", Urls));
        }
    }
}